package com.proyecto.freetoursvcl;

import java.net.HttpURLConnection;
import java.util.ArrayList;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONObject;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.Toast;

import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;

public class AlmacenMonumentosSW extends AsyncTask<Integer, Void, ArrayList<Monumento>> {
	 
	private ProgressDialog progreso;
	private Context context;
 	private SharedPreferences preferencias;
 	private int radio;
 	private long latitud;
 	private long longitud;
 	private String filtros;

    public AlmacenMonumentosSW(Context context, LatLng posicion) {
          
    	  this.filtros = "";
    	  this.context = context;
		  preferencias = context.getSharedPreferences("com.proyecto.freetoursvcl_preferences", Context.MODE_PRIVATE);

//		  Log.d("ServicioWeb","LATITUD-ANTES:" + posicion.latitude);
//		  Log.d("ServicioWeb","LONGITUD-ANTES:" + posicion.longitude);
		  
		  latitud = (long)(posicion.latitude * 1000000);
		  longitud = (long)(posicion.longitude * 1000000); 
		  
//		  Log.d("ServicioWeb","LATITUD:" + latitud);
//		  Log.d("ServicioWeb","LONGITUD:" + longitud);			      
  }

    @Override 
    protected synchronized void onPreExecute() {
        progreso = new ProgressDialog(context);
        progreso.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progreso.setMessage("Obteniendo datos del Servidor...");
        progreso.setCancelable(false);
        progreso.show();
    }
    
    @Override 
    protected ArrayList<Monumento> doInBackground(Integer... n) {
        return listaMonumentos(n[0]);
    }
	
    @Override 
    protected void onPostExecute(ArrayList<Monumento> auxiliar) {
        progreso.dismiss();
        Monumentos.vectorMonumentos= auxiliar;
        ubicarMonumentos();
        
    }
    
    @Override 
    protected void onCancelled() {
        progreso.dismiss();
        Toast.makeText(context, "Error accediendo al Servicio2", Toast.LENGTH_SHORT).show();
    }
    
    public synchronized ArrayList<Monumento> listaMonumentos(int radio) {
		  
      this.radio = radio;
      recuperarFiltros();
	  Log.d("ServicioWeb","RADIO:" + this.radio);
	  Log.d("ServicioWeb","FILTROS:" + filtros);
      ArrayList<Monumento> monumentos = new ArrayList<Monumento>();
    
      Log.d("ServicioWeb", "INICIO!!!!!!!!!!!!!!!!!");
      HttpClient httpClient = new DefaultHttpClient();
      
      HttpGet del = new HttpGet("http://mapas.valencia.es/lanzadera/puntoInteres/monumentos?radio="
    		  + radio + "&lang=es&lat=" + latitud + "&lon=" + longitud + "&filtros=" + filtros);
                    
      try
      {
          del.setHeader("content-type", "application/json");
          del.setHeader("Authorization", "Basic anRvbWFzOnZzYXpyaWFy");
          
          //del.setHeader("Host", "http://mapas.valencia.es");
          del.setHeader("User-Agent", "Mozilla/4.0");
          Log.d("Servicio", "empizo la conexion");
          //del.setHeader("Accept", "application/json");
          //del.setHeader("Accept-Charset", "utf-8");
      	    
  	    HttpResponse resp = httpClient.execute(del);        	                        
  	    Log.d("Servicio", "STATUSCODE:"+ String.valueOf(resp.getStatusLine().getStatusCode()));

  	    switch (resp.getStatusLine().getStatusCode()) {
  	    
  	    case HttpURLConnection.HTTP_OK:
  	    	
  	    	String respStr = EntityUtils.toString(resp.getEntity());
              JSONArray respJSON = new JSONArray(respStr);
      	    Log.d("Servicio", "RESPUESTA:"+ respStr);
          
              for(int i=0; i<respJSON.length(); i++)
              {
              	Log.d("Servicio", "Valor i:"+ String.valueOf(i));
              	JSONObject obj = respJSON.getJSONObject(i);
       
                  Monumento monumento = new Monumento();
                  monumento.setNombre(obj.getString("titulo"));
                  monumento.setDireccion(obj.getString("texto"));
                  monumento.setFoto(obj.getString("imagen"));
                  double latitud = (double)obj.getInt("lat");
                  latitud = latitud / 1000000;
                  double longitud = (double)obj.getInt("lon");
                  longitud = longitud / 1000000;
                  GeoPunto posicion = new GeoPunto(latitud,longitud);
                  monumento.setPosicion(posicion);
                  monumento.setDistancia(obj.getInt("distancia"));
                  monumento.setTipo(obj.getInt("tipo"));
                  
                  JSONArray respJSONacc = new JSONArray(obj.getString("acciones"));
                  
                  for(int j=0; j<respJSONacc.length(); j++)
                  {
                  	JSONObject objAcc = respJSONacc.getJSONObject(j);
                  	Acciones accion = new Acciones();
                  	accion.setNombreAc(objAcc.getString("nombre"));
                  	accion.setTipoAc(objAcc.getString("tipo"));
                  	accion.setUri(objAcc.getString("uri"));
                  	monumento.anyadeAccion(accion);
                  }
                                         
                  monumentos.add(monumento);  
              	Log.d("Servicio", "Monumento:"+ monumento.toString());
              }
              break;
  	    case HttpURLConnection.HTTP_NOT_FOUND:
  	    	Toast.makeText(context, "Servicio no disponible en estos momentos", Toast.LENGTH_SHORT).show();
  	    	break;
  	    default:	
  	    	Toast.makeText(context, "Error accediendo al Servicio", Toast.LENGTH_SHORT).show();
  	    	break;
	    	
  	    }
  	    
          return monumentos;
      }
      catch(Exception ex)
      {
              Log.e("ServicioRest","Error!", ex);
              return monumentos;
      }

	}
    public void ubicarMonumentos() {

   	 	MainActivity.mapa.clear();
    	for (Monumento monumento : Monumentos.vectorMonumentos) {

   	        GeoPunto p = monumento.getPosicion();
   	        if (p != null && p.getLatitud() != 0) {
   	            MainActivity.mapa.addMarker(new MarkerOptions()
   	                 .position(new LatLng(p.getLatitud(), p.getLongitud()))
//   	                 .title(monumento.getNombre()).snippet(monumento.getDireccion())
//   	                 .icon(BitmapDescriptorFactory.fromResource(R.drawable.ic_city))
   	                 .title(monumento.getNombre()).snippet("Pulse aquí para obtener más infomación...")
   	                 .icon(BitmapDescriptorFactory.fromResource(R.drawable.monumentos_turisticos))
   	       	         .anchor(0.5f, 0.5f));
   	        }
   	    }

    }
    
    public void recuperarFiltros() {
    	
    	if (preferencias.getBoolean("Otros_Tipos", true)) {
    		filtros = filtros + "0,";
    	}
    	if (preferencias.getBoolean("Romana", true)) {
    		filtros = filtros + "1,";
    	}
    	if (preferencias.getBoolean("Musulmana", true)) {
    		filtros = filtros + "2,";
    	}
    	if (preferencias.getBoolean("Medieval", true)) {
    		filtros = filtros + "3,";
    	}
    	if (preferencias.getBoolean("Renacentista", true)) {
    		filtros = filtros + "4,";
    	}
    	if (preferencias.getBoolean("Barroca", true)) {
    		filtros = filtros + "5,";
    	}
    	if (preferencias.getBoolean("Ilustrada", true)) {
    		filtros = filtros + "6,";
    	}
    	if (preferencias.getBoolean("Decimonónica", true)) {
    		filtros = filtros + "7,";
    	}
    	if (preferencias.getBoolean("Entre_dos_Siglos", true)) {
    		filtros = filtros + "8,";
    	}
    	if (preferencias.getBoolean("Siglo_XX", true)) {
    		filtros = filtros + "9,";
    	}
    	if (preferencias.getBoolean("Tercer_Milenio", true)) {
    		filtros = filtros + "10,";
    	}
    	if (preferencias.getBoolean("Marinera", true)) {
    		filtros = filtros + "11,";
    	}
    	if (preferencias.getBoolean("Natural", true)) {
    		filtros = filtros + "12,";
    	}
    	
    }
    
}
