package com.proyecto.freetoursvcl;


import java.text.DecimalFormat;
import java.util.Timer;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnInfoWindowClickListener;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;


public class MainActivity extends FragmentActivity implements LocationListener, OnInfoWindowClickListener,
													OnSeekBarChangeListener{

	public static Monumentos almacen;
	private SeekBar ControlDistancia;
	private TextView SalidaDistancia;
	private DecimalFormat formateador; 
    
	private static final long DOS_MINUTOS = 1 * 60 * 1000;
    final static String TAG = "Practica";
    private static LatLng posicionActual = new LatLng(0,0);
    protected static GoogleMap mapa;
    private LocationManager manejador;
    private Location mejorLocaliz;
    private boolean ActivarServicio = true;
    private SharedPreferences preferencias;
    //Temporizador
    Timer timer;
    MyTimerTask myTimerTask;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            setContentView(R.layout.activity_main);
            //Definimos la barra de progreso
            ControlDistancia = (SeekBar) findViewById(R.id.seekBar1);
            ControlDistancia.setOnSeekBarChangeListener(this);
            SalidaDistancia = (TextView) findViewById(R.id.seleccion);
            formateador = new DecimalFormat("#.#");
            ///Definimos las caracteristicas del maps que vamos a mostrar
            mapa = ((SupportMapFragment) getSupportFragmentManager()
                              .findFragmentById(R.id.map)).getMap();

          	mapa.setMapType(GoogleMap.MAP_TYPE_HYBRID);
            mapa.setMyLocationEnabled(true);
            mapa.getUiSettings().setZoomControlsEnabled(false);
            mapa.getUiSettings().setCompassEnabled(true);
            ///Declaramos el manejador y buscamos una primera ubicaci�n para los dos proveedores del servicio que vamos a utilizar
            manejador = (LocationManager) getSystemService(LOCATION_SERVICE);
            if (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
                   actualizaMejorLocaliz(manejador.getLastKnownLocation(LocationManager.GPS_PROVIDER));
            }
            if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
                   actualizaMejorLocaliz(manejador.getLastKnownLocation(LocationManager.NETWORK_PROVIDER));
            }
            
            synchronized (this) {
            	almacen = new Monumentos();
	            AlmacenMonumentosSW tarea = new AlmacenMonumentosSW(this,posicionActual);
	            tarea.execute(ControlDistancia.getProgress());
            }
            
            mapa.setOnInfoWindowClickListener(this);
            
            preferencias = this.getSharedPreferences("com.proyecto.freetoursvcl_preferences", Context.MODE_PRIVATE);
            
            if (preferencias.getBoolean("notificacion", true)) {
                
            	activarTemporizador();
        	}
            
    }
    
    ///Cuando la actividad pasa a estar activa actualizamos los proveedores de servicios y comprobamos
    @Override 
    protected void onResume() {
        super.onResume();
        activarProveedores();
        //Si el usuario ha desacticado las notificaciones cancelamos el temporizador
        if (!preferencias.getBoolean("notificacion", true)) {
            if (timer!=null){
                timer.cancel();
                timer = null;
            }
    	}else{
    		//Si el usuario ha vuelto a activar las notificaciones activamos el temporizador
    		if (timer==null){
    			activarTemporizador();
    		}  			
    	}
	 }
    
     private void activarTemporizador(){
         
    	if(timer != null){
             timer.cancel();
        }
            
        timer = new Timer();
        myTimerTask = new MyTimerTask(this,posicionActual);
        // delay 60 seg. 60000ms, repeat in 30 segundos
        timer.schedule(myTimerTask, 60000, 30000);
 	
     }
	 ///Las actualizaciones de los proveedores se programan cada 20 y 10 segundos y con 5 y 10 metros minimos. 
	 private void activarProveedores() {
	        if (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
	              manejador.requestLocationUpdates(LocationManager.GPS_PROVIDER,
	                                   20 * 1000, 5, this);
	        }
	        if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
	              manejador.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
	                                   10 * 1000, 10, this);
	        }
	 }
	 //Cuando la actividad deje de estar activa, eliminamos las actualizaciones 
	 @Override 
	 protected void onPause() {
	        super.onPause();
	        manejador.removeUpdates(this);
	 }
    
    /// Obtenemos la mejor posici�n	 
	 private void actualizaMejorLocaliz(Location localiz) {
	        if (localiz != null && (mejorLocaliz == null
	        || localiz.getAccuracy() < 2*mejorLocaliz.getAccuracy()
	        || localiz.getTime() - mejorLocaliz.getTime() > DOS_MINUTOS)) {
	              Log.d(TAG, "Nueva mejor localizaci�n");
	              mejorLocaliz = localiz;
	              posicionActual = new LatLng(localiz.getLatitude(), localiz.getLongitude());
	              mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(posicionActual,15));

	        }
	 }
	 ///Metodos de la interfaz LocationListener
	 @Override 
	 public void onLocationChanged(Location location) {
	       Log.d(TAG, "Nueva localizaci�n: "+location);
	       actualizaMejorLocaliz(location);
	}
	 
	@Override 
	public void onProviderDisabled(String proveedor) {
	       Log.d(TAG, "Se deshabilita: "+proveedor);
	       activarProveedores();
	}
	 
	@Override 
	public void onProviderEnabled(String proveedor) {
	       Log.d(TAG, "Se habilita: "+proveedor);
	       activarProveedores();
	}
	 
	@Override
	public void onStatusChanged(String proveedor, int estado, Bundle extras) {
	       Log.d(TAG, "Cambia estado: "+proveedor);
	       activarProveedores();
	}

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. 
    	
        switch (item.getItemId()) {

        case R.id.menu_listado:
	
	          lanzarListado(null);
	          break;
        case R.id.acerca:

            lanzarAcercaDe(null);
            break;
        case R.id.conf:

            lanzarPreferencias(null);
            break;
        }

        return true; 
    }
    public void lanzarAcercaDe(View view){
        
    	Intent i = new Intent(this, AcercaDe.class);
    	startActivity(i);
	
    }
    
    public void lanzarPreferencias(View view){
    	
    	Intent i = new Intent(this, Preferencias.class);
    	startActivity(i);  	
    }
    
    public void lanzarListado(View view) {
        Intent i = new Intent(this, Listado.class);
        startActivity(i);
    }

    @Override public void onInfoWindowClick(Marker marker) {
        for (int id=0; id < Monumentos.vectorMonumentos.size(); id++){ 
            if (Monumentos.vectorMonumentos.get(id). getNombre().equals(marker.getTitle())){
                Intent intent = new Intent(this, VistaMonumento.class);
                intent.putExtra("Tipo", "Mapa");
                intent.putExtra("id", (long)id);
                startActivity(intent);
                break;
            }
        }
    }

	@Override
	public void onProgressChanged(SeekBar seekBar, int progress,
			boolean fromUser) {
		if (progress > 999){
			float res = (float)progress;
			SalidaDistancia.setText(String.valueOf(formateador.format(res / 1000)) + "Km");
		}else
			SalidaDistancia.setText(String.valueOf(progress) + "m");
		
	}

	@Override
	public void onStartTrackingTouch(SeekBar seekBar) {
		//Toast.makeText(this, "empiezo a modificar", Toast.LENGTH_SHORT).show();
		
	}

	@Override
	public void onStopTrackingTouch(SeekBar seekBar) {
		//Toast.makeText(this, "llamo a redibujar", Toast.LENGTH_SHORT).show();
		AlmacenMonumentosSW tarea = new AlmacenMonumentosSW(this,posicionActual);
        tarea.execute(ControlDistancia.getProgress());
		
	}
	
	@Override
	protected void onSaveInstanceState(Bundle guardarEstado) {
	       super.onSaveInstanceState(guardarEstado);
	       guardarEstado.putBoolean("indServicio", ActivarServicio);

	}
	 
	@Override
	protected void onRestoreInstanceState(Bundle recEstado) {
	       super.onRestoreInstanceState(recEstado);
	       ActivarServicio = recEstado.getBoolean("indServicio");
	}

	static LatLng getPosicionActual() {
		return posicionActual;
	}

    
}